#include "3dView.h"
#include "MapSegments.h"
#include "TrackEdit.h"
#include <math.h>

void C3DView::FreeObjects(RenderObject *o)
{
	RenderObject *Next;
	while(o)
	{
		Next = o->Next;
		delete o;
		o = Next;
	}
}

void C3DView::FreeSegments(RenderSegment *Head)
{
	RenderSegment *Next;
	while(Head)
	{
		Next = Head->Next;
		delete Head;
		Head = Next;
	}
}

void C3DView::FreeCorners(RenderCorner *Head)
{
	RenderCorner *Next;
	while(Head)
	{
		Next = Head->Next;
		delete Head;
		Head = Next;
	}
}

void C3DView::FixCorners(int p)
{
	FreeCorners(RenderPaths[p].CornHead); RenderPaths[p].CornHead = NULL;

	/* really just duplicate global corner stuff */
	CornerNode *C = Paths[p].CornerList;
	RenderCorner **NewCorner = &RenderPaths[p].CornHead;

	*NewCorner = new RenderCorner;
	(*NewCorner)->y = Paths[p].StartY;
	(*NewCorner)->Curvature = 0;
	NewCorner = &(*NewCorner)->Next;

	while(C)
	{
		*NewCorner = new RenderCorner;
		(*NewCorner)->y = C->y;
		(*NewCorner)->Curvature = C->Curvature;
		NewCorner = &(*NewCorner)->Next;
		C = C->Next;
	}
	*NewCorner = NULL;
}

void C3DView::FixObjects(int p)
{
	FreeObjects(RenderPaths[p].ObjHead); RenderPaths[p].ObjHead = NULL;

	/* position and link objects */
	RenderObject **Obj = &RenderPaths[p].ObjHead;
	Object *SrcObject = Paths[p].ObjList;
	while(SrcObject)
	{
		/* find meaningful segment */
		RenderSegment *SSeg = RenderPaths[p].SegHead;
		while(SSeg->Next && SrcObject->y >= SSeg->Next->ArcStart)
			SSeg = SSeg->Next;

		*Obj = new RenderObject;
		(*Obj)->WorldX = SrcObject->x;
		(*Obj)->ImageId = SrcObject->ImageId;
		(*Obj)->ArcPos = SrcObject->y;

		float Ratio = (SrcObject->y - SSeg->ArcStart) / (SSeg->Next->ArcStart - SSeg->ArcStart);
		(*Obj)->OffsetY = Ratio*SSeg->y;
		(*Obj)->OffsetZ = Ratio*SSeg->z;
		(*Obj)->ParentSegment = SSeg;

		/* find meaningful corner */
		RenderCorner *SCorn = RenderPaths[p].CornHead;
		if(SCorn)
		{
			while(SCorn->Next && SCorn->Next->y < SrcObject->y)
				SCorn = SCorn->Next;
		}
		if(SCorn && SCorn->y < SrcObject->y)
			(*Obj)->ParentCorner = SCorn;
		else
			(*Obj)->ParentCorner = NULL;

		Obj = &(*Obj)->Next;
		SrcObject = SrcObject->Next;
	}
	*Obj = NULL;
}

void C3DView::FixStarts()
{
	float CumulativeY = 0;
	float CumulativeZ = 0;
	for(int c = 0; c < 8; c++)
	{
		RenderSegment *Seg;
		if( Paths[ SectionIndices[c] ].Brother != -1)
		{
			float CZ = CumulativeZ;
			float CY = CumulativeY;

			Seg = RenderPaths[ Paths[SectionIndices[c]].Brother ].SegHead;
			while(Seg && Seg->Next)
			{
				Seg->CumulativeY = CY;
				Seg->CumulativeZ = CZ;
				CY += Seg->y;
				CZ += Seg->z;
				Seg = Seg->Next;
			}
		}

		Seg = RenderPaths[ SectionIndices[c] ].SegHead;
		while(Seg && Seg->Next)
		{
			Seg->CumulativeY = CumulativeY;
			Seg->CumulativeZ = CumulativeZ;
			CumulativeY += Seg->y;
			CumulativeZ += Seg->z;
			Seg = Seg->Next;
		}
		
	}
}

void C3DView::FixElevation(int p)
{
	/* just do first path for now */
	FreeSegments(RenderPaths[p].SegHead); RenderPaths[p].SegHead = NULL;

	/* seed positions */
	ElevationNode *ENode = Paths[p].ElevationList;
	RenderSegment **Seg = &RenderPaths[p].SegHead;
	while(ENode->Next)
	{
		for(int c = 0; c < 100; c++)
		{
			*Seg = new RenderSegment;
			EvaluateElevation(ENode, ENode->Next, (float)c / 100.0f, &(*Seg)->y, &(*Seg)->z);
			Seg = &(*Seg)->Next;
		}
		ENode = ENode->Next;
	}
	*Seg = new RenderSegment;
	EvaluateElevation(ENode, ENode, 0, &(*Seg)->y, &(*Seg)->z);
	(*Seg)->Next = NULL;

	/* make relative */
	RenderSegment *ParseSeg = RenderPaths[p].SegHead;
	float ArcStart = Paths[p].StartY;

	while(ParseSeg->Next)
	{
		ParseSeg->CumulativeY = ParseSeg->y;
		ParseSeg->CumulativeZ = -ParseSeg->z;
		ParseSeg->y = ParseSeg->Next->y - ParseSeg->y;
		ParseSeg->z = ParseSeg->z - ParseSeg->Next->z;
		
		ParseSeg->ArcStart = ArcStart;
		ArcStart += sqrt(ParseSeg->y*ParseSeg->y + ParseSeg->z*ParseSeg->z);
		ParseSeg->Angle = atan2(ParseSeg->y, ParseSeg->z);

		ParseSeg = ParseSeg->Next;
	}
	ParseSeg->ArcStart = ArcStart;

	FixStarts();
}
