#ifndef __3DVIEW_H
#define __3DVIEW_H

class C3DView
{
	public:
		C3DView();
		~C3DView();

		void SetRenderArea(float x1, float y1, float x2, float y2);
		void FixElevation(int Path);
		void FixCorners(int Path);
		void FixObjects(int Path);

		void DrawTrackPOV(int section, float x, float y, float z);
		void Draw(int section, float x, float y, float z);

 		float GetSkew(int path, float starty, float endy);

	private:
		void FixStarts();
	
		struct RenderSegment
		{
			float Angle;

			float ArcStart;
			float y, z;

			float ScrLowY;

			float CumulativeY, CumulativeZ;

			RenderSegment *Next;
		};

		class RenderCorner
		{
			public:
				float y, Curvature;
				float Skew, SkewMultiplier, CurveBase;

				void Effect(float Curvature, float CurveOffset, float &Skew, float &SkewMultiplier);

			RenderCorner *Next;
		};

		struct RenderObject
		{
			float WorldX, OffsetY, OffsetZ;
			float ArcPos;
			int ImageId;

			RenderSegment *ParentSegment;
			RenderCorner *ParentCorner;
			RenderObject *Next;
		};

		struct PathSet
		{
			RenderSegment *SegHead;
			RenderObject *ObjHead;
			RenderCorner *CornHead;
		} RenderPaths[26];

		/* stuff specifically used for line by line rendering of a frame */
		bool DrawPathFloor(int path, float x, float y, float z, RenderSegment *Seg, RenderCorner *Corner, float Curvature);
		void DrawPathObjects(int path, float x, float y, float z);
		void DrawPath(int path, float x, float y, float z, RenderSegment *Seg);

		int Lines;
		float DrawY;
		float Skew, SkewMultiplier;
		float CurveBase;
		float ArcBase;

		void EffectCurve(float Curvature, float CurveOffset);
		void AddScanline(float Distance, float SurfaceY, int path, float x);

		/* memory dealloc procs */
		void FreeSegments(RenderSegment *s);
		void FreeObjects(RenderObject *o);
		void FreeCorners(RenderCorner *c);

		/* general stuff */
		int Steps;
		float StepSize;
		float *CosTable, *CastAngles, *SizeMultipliers;
		float x1, y1, x2, y2;
		float CentreX;
};

extern C3DView *SceneViewer;

#endif
