#ifndef __BITMAP_H
#define __BITMAP_H

#include "GLFoundation.h"
#include "SDL.h"

#define	BMP_HFLIP	0x4000
#define BMP_VFLIP	0x8000

void DisableTextures();

class CBitmap
{
	public:
		~CBitmap();
		CBitmap();

		bool Load(char *name, bool SmoothScale = false);
		void Activate();

		void Backup();
		void Restore();
		
		int Width() { return ImageWidth; }
		int Height() { return ImageHeight; }

		bool PointOver(float x, float y);
		void Draw(float x1, float y1, float x2, float y2, int flags);

	private:
	
		/* OpenGL stuff */
		GLuint TexId;
		static GLuint ActiveTexId;
		unsigned char *ImageData;

		/* image stats */
		int ImageWidth, ImageHeight;
		bool SmoothScale;
		int TexTarget;

		/* bit mask */
		Uint32 *Mask;
		int MaskPitch;
};

class CBitmapSet
{
	public:
		void Backup();
		void Restore();

		void Register(CBitmap *);
		void DeRegister(CBitmap *);
		
		~CBitmapSet();
		CBitmapSet();

	private:
		struct BitmapPtr
		{
			CBitmap *Data;
			BitmapPtr *Next;
		} *Head;
};

extern CBitmapSet *BitmapSet;

#endif
