#include "GLFoundation.h"
#include "SystemDriver.h"
#include <string.h>

char *ExtString;
void SeedExtensionSearch()
{
	ExtString = (char *)glGetString(GL_EXTENSIONS);
}

bool ExtensionSearch(char *extName)
{
	char *p = ExtString;
	char *end;
	int extNameLen;

	extNameLen = strlen(extName);
	end = p + strlen(p);

	while (p < end)
	{
		int n = strcspn(p, " ");
		if ((extNameLen == n) && (strncmp(extName, p, n) == 0))
			return GL_TRUE;
		p += (n + 1);
	}

	return GL_FALSE;
}

CGLFoundation GLFoundation;
int GLCapabilities;
float ScrW, ScrH;
int FrameNum;
int GLRectangleTexTarget;

#define HFOV 0.5f

//7735026919

void CGLFoundation::SetMode(int w, int h, bool fullscreen)
{
	SystemDriver->SetMode(w, h, FullScreen = fullscreen);
	w = SystemDriver->NewWidth(); h = SystemDriver->NewHeight();
	glViewport(0, 0, w, h);

	/* enable depth testing & face culling */
	glEnable(GL_DEPTH_TEST);
	glClearDepth(1.0f);
	glDepthFunc(GL_LEQUAL);

//	glEnable(GL_CULL_FACE);
//	glCullFace(GL_BACK);
//	glFrontFace(GL_CCW);

	/* enable blending with the ordinary function*/
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	
	/* enable alpha testing to discard depth writes for low opaque pixels */
	glAlphaFunc(GL_GEQUAL, 8.0f / 256.0f);
	glEnable(GL_ALPHA_TEST);

	/* hint for nicest perspective correction */
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	/* set default 60 degree horizontal FOV projection matrix */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	float bound = (float)h / (float)w;
	glFrustum(-HFOV, HFOV, -bound*HFOV, bound*HFOV, 0.5, 2000);
	glScalef( 1.0f, -1.0f, -1.0f);

	ScrW = HFOV*2.0f;
	ScrH = bound*HFOV*2.0f;

	/* load identity to modelview */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	/* get extensions */
		GLCapabilities = 0;
		SeedExtensionSearch();

		/* compressed textures */
		if(ExtensionSearch("GL_ARB_texture_compression"))
		{
			GLCapabilities |= GLCAP_COMPRESSED_TEXTURE;
			glHint(GL_TEXTURE_COMPRESSION_HINT, GL_NICEST);
		}

		/* non power of two textures */
		if(ExtensionSearch("GL_ARB_texture_non_power_of_two"))
			GLCapabilities |= GLCAP_NON_POWER_OF_TWO;

		/* texture rectangles */
		if(ExtensionSearch("GL_NV_texture_rectangle"))
		{
			GLCapabilities |= GLCAP_TEXTURE_RECTANGLE;
			GLRectangleTexTarget = GL_TEXTURE_RECTANGLE_NV;
		}
		if(ExtensionSearch("GL_ARB_texture_rectangle"))
		{
			GLCapabilities |= GLCAP_TEXTURE_RECTANGLE;
			GLRectangleTexTarget = GL_TEXTURE_RECTANGLE_ARB;
		}
		if(ExtensionSearch("GL_EXT_texture_rectangle"))	/* prefer ext to arb for ATI reasons */
		{
			GLCapabilities |= GLCAP_TEXTURE_RECTANGLE;
			GLRectangleTexTarget = GL_TEXTURE_RECTANGLE_EXT;
		}
}

void CGLFoundation::Flip()
{
	SystemDriver->Flip();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	FrameNum++;
}


void CGLFoundation::SetScissor(float x1, float y1, float x2, float y2)
{
	/* map to window co-ordinates */
	glEnable(GL_SCISSOR_TEST);
	glScissor(
		(int)((x1+ScrW)*SystemDriver->NewWidth() / (2*ScrW)), 
		(int)((ScrH-y2)*SystemDriver->NewHeight() / (2*ScrH)),
		(int)((x2-x1)*SystemDriver->NewWidth() / (2*ScrW)),
		(int)((y2-y1)*SystemDriver->NewHeight() / (2*ScrH))
	);
}

int CGLFoundation::GetNumScanlines(float y1, float y2)
{
	return (int)(((y2 - y1)*SystemDriver->NewHeight()) / (2*ScrH));
}

void CGLFoundation::DisableScissor()
{
	glDisable(GL_SCISSOR_TEST);
}
