#include "TrackEdit.h"

/*

	IMAGEBROWSER

*/

CImageBrowser::CImageBrowser()
{
	AddWidget(&PositionSlider);
	PositionSlider.SetValue(0);
	Selected = -1;
}

void CImageBrowser::SetPos(float x1, float y1, float x2, float y2)
{
	CWidgetBase::SetPos(x1, y1, x2, y2);

	PositionSlider.SetPos(x1, y2, x2, y2);
	PositionSlider.SizeToFit(OT_BOTTOM);
}

void CImageBrowser::SetImageSet(CImageArray *imga)
{
	IArray = imga;
}

bool CImageBrowser::Message(DialogueMessage *Msg)
{
	CDialogueBase::Message(Msg);

	switch(Msg->Type)
	{
		default: break;
		case DialogueMessage::CLICK:
		{
			if(Msg->x < x1) return false;
			if(Msg->x > x2) return false;
			if(Msg->y < y1) return false;
			if(Msg->y > y2) return false;

			float cy, cx;
			int num = base;

			/* dirty, dirty, dirty */
			cy = y1 + (1.0f / 40.0f) + 0.025f;
			while(cy+0.1f < (y2 - PositionSlider.Height()))
			{
				cx = x1 + 0.05f;
				while(cx+0.1f < x2)
				{
					if(num < IArray->GetNumImages())
					{
						if(
							Msg->x >= cx && Msg->y >= cy &&
							Msg->x <= cx+0.1f && Msg->y <= cy+0.1f)
						{
							if(Parent)
							{
								DialogueMessage NewMsg;
								NewMsg.Type = DialogueMessage::DOTASK;
								NewMsg.Data.Task.Num = id;
								NewMsg.Data.Task.Data = num;
								Parent->Message(&NewMsg);
							}
							Selected = num;
							return true;
						}
					}

					cx += 0.15f;
					num++;
				}
				cy += 0.15f;
			}

			/* if here then user possibly clicked nothing */
			if(Msg->y < y2 - PositionSlider.Height())
			{
				if(Parent)
				{
					DialogueMessage NewMsg;
					NewMsg.Type = DialogueMessage::DOTASK;
					NewMsg.Data.Task.Num = id;
					NewMsg.Data.Task.Data = -1;
					Parent->Message(&NewMsg);
				}
				Selected = -1;

				return true;
			}
		}
		return false;
		case DialogueMessage::DRAW:
		{
			float cy;
			float cx;
			int slots = 0, cbase;

			/* count slots */
			cy = y1 + (1.0f / 40.0f) + 0.025f;
			while(cy+0.1f < (y2 - PositionSlider.Height()))
			{
				cx = x1 + 0.05f;	
				while(cx+0.1f < x2)
				{
					cx += 0.15f;
					slots++;
				}
				cy += 0.15f;
			}

			/* decide where we are in the list */
			if(IArray->GetNumImages() > slots)
				base = (int)(PositionSlider.GetValue()*(IArray->GetNumImages()-slots) + 0.5f);
			else
				base = 0;
			cbase = base;

			/* draw images */
			cy = y1 + (1.0f / 40.0f) + 0.025f;
			while(cy+0.1f < (y2 - PositionSlider.Height()))
			{
				cx = x1 + 0.05f;
				while(cx+0.1f < x2)
				{
					CBitmap *img = IArray->GetImage(cbase&TILE_MASK);
					if(img)
					{
						float R = (float)img->Width() / img->Height();
						if(R > 1.0f)
						{
							/* wider than tall */
							img->Draw(cx, cy+0.05f-0.05f/R, cx+0.1f, cy+0.05f+0.05f/R, FlipFlags);
						}
						else
						{
							/* taller than wide */
							img->Draw(cx+0.05f-0.05f*R, cy, cx+0.05f+0.05f*R, cy+0.1f, FlipFlags);
						}

						if(cbase == Selected)
						{
							int Ratio = SDL_GetTicks()&2047;
							if(SDL_GetTicks()&2048) Ratio ^= 2047;
							glColor4f(1, 1, 1, ((float)Ratio / 4094.0f));
							DisableTextures();
							glBegin(GL_QUADS);
								glVertex2f(cx, cy);
								glVertex2f(cx + 0.1f, cy);
								glVertex2f(cx + 0.1f, cy+0.1f);
								glVertex2f(cx, cy+0.1f);
							glEnd();
							glColor3f(1, 1, 1);
						}
					}

					cx += 0.15f;
					cbase++;
				}
				cy += 0.15f;
			}
		}
		return false;
	}
	return false;
}
