#include "SystemDriver.h"
#include "GLFoundation.h"
#include "DialogueBase.h"
#include "SegmentSelect.h"
#include <unistd.h>
#include "Bitmap.h"
#include "TextWriter.h"
#include "MapSegments.h"
#include "ImageArray.h"
#include "Stack.h"
#include "3dView.h"

CDialogueBase *DialogueStack[10];
int DSReadPtr = 0, DSWritePtr = 0;

void PushDialogue(CDialogueBase *db)
{
	DSWritePtr++;
	DialogueStack[DSWritePtr] = db;
}

int OperationStack[10];
int OSWritePtr = 0;
void DoStackOp(int op)
{
	OperationStack[OSWritePtr] = op;
	OSWritePtr++;
}

int main(int argc, char *argv[])
{
	SystemDriver = new CSystemDriver;

	bool FullScr = false;
	int W = 640, H=480;

	FILE *prefs;
	if(prefs = fopen("editor.prefs", "rb"))
	{
		W = fgetc(prefs); W |= fgetc(prefs) << 8;
		H = fgetc(prefs); H |= fgetc(prefs) << 8;
		fclose(prefs);
	}

	GLFoundation.SetMode(W, H, false);
	bool ScrChange = false;
	int OldButtons = 0;

	BitmapSet = new CBitmapSet;
	TextWriter = new CTextWriter;
	MaterialArray = new CImageArray("tiles/tile");
	ObjectArray = new CImageArray("objects/object");
	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
	SceneViewer = new C3DView;
	LoadSegments();

	DialogueStack[DSReadPtr] = new CSegmentSelect;
	DialogueMessage NewMsg;
	NewMsg.Type = DialogueMessage::ENTER;
	DialogueStack[DSReadPtr]->Message(&NewMsg);

	while(!SystemDriver->QuitRequested())
	{
		GLFoundation.Flip();
		bool Resize = SystemDriver->Resized();
		int W = SystemDriver->NewWidth(), H = SystemDriver->NewHeight();

		const Uint8 *Keys = SystemDriver->GetKeyState();

		if(Keys[SDLK_RETURN])
		{
			if(!ScrChange)
			{
				FullScr = !FullScr; Resize = true;
				ScrChange = true;
			}
		}
		else
			ScrChange = false;

		if(Resize)
		{
			NewMsg.Type = DialogueMessage::EXIT;
			DialogueStack[DSReadPtr]->Message(&NewMsg);
			BitmapSet->Backup();

			GLFoundation.SetMode(W, H, FullScr);

			BitmapSet->Restore();
			NewMsg.Type = DialogueMessage::ENTER;
			DialogueStack[DSReadPtr]->Message(&NewMsg);
		}

		int c = 20;
		while(c--)
		{
			if(!SDL_PollEvent(NULL)) SDL_Delay(5);
			else break;
		}

		int mx, my;
		int Buttons;
		Buttons = SDL_GetMouseState(&mx, &my);
		float MouseX, MouseY;

		MouseX = (float)((mx << 1) - W) / W;
		MouseY = (float)((my << 1) - H) / W;
		NewMsg.x = MouseX;
		NewMsg.y = MouseY;
		NewMsg.Buttons = Buttons;

		NewMsg.Type = DialogueMessage::MOUSEMOVE;
		DialogueStack[DSReadPtr]->Message(&NewMsg);

		if(MouseScroll)
		{
			NewMsg.Type = DialogueMessage::SCROLL;
			NewMsg.Data.Scroll = MouseScroll;
			DialogueStack[DSReadPtr]->Message(&NewMsg);
			MouseScroll = 0;
		}

		if(Buttons^OldButtons)
		{
			NewMsg.Type = DialogueMessage::CLICK;
			NewMsg.Data.ClickChange = Buttons^OldButtons;
			DialogueStack[DSReadPtr]->Message(&NewMsg);
		}
		OldButtons = Buttons;

		int NumKeys = SystemDriver->GetNumBufferedKeys();
		while(NumKeys--)
		{
			NewMsg.Type = DialogueMessage::KEYPRESS;
			NewMsg.Data.Key = SystemDriver->GetBufferedKey();
			DialogueStack[DSReadPtr]->Message(&NewMsg);
		}

		NewMsg.Type = DialogueMessage::DRAW;

		glPushMatrix();

			glTranslatef(0, 0, 1);
			DialogueStack[DSReadPtr]->Message(&NewMsg);

		glPopMatrix();

		if(OSWritePtr)
		{
			while(OSWritePtr--)
			{
				NewMsg.Type = DialogueMessage::EXIT;
				DialogueStack[DSReadPtr]->Message(&NewMsg);
				switch(OperationStack[OSWritePtr])
				{
					case SI_DESCEND:
						DSReadPtr++;
					break;
					case SI_ASCEND:
						delete DialogueStack[DSReadPtr];
						DSReadPtr--;
					break;
				}
				NewMsg.Type = DialogueMessage::ENTER;
				DialogueStack[DSReadPtr]->Message(&NewMsg);
			}
			OSWritePtr = 0;
		}
		DSWritePtr = DSReadPtr;
	}

	NewMsg.Type = DialogueMessage::EXIT;
	DialogueStack[DSReadPtr]->Message(&NewMsg);

	if(prefs = fopen("editor.prefs", "wb"))
	{
		W = SystemDriver->NewWidth();
		H = SystemDriver->NewHeight();
		putc(W&0xff, prefs); fputc((W >> 8)&0xff, prefs);
		putc(H&0xff, prefs); fputc((H >> 8)&0xff, prefs);
		fclose(prefs);
	}

	delete SceneViewer;
	SaveSegments();
	delete MaterialArray;
	delete ObjectArray;
	while(1)
	{
		delete DialogueStack[DSReadPtr];
		if(!DSReadPtr) break;
		DSReadPtr--;
	}
	delete TextWriter;
	delete BitmapSet;
	delete SystemDriver;

    return 0;
}
