#include "SegmentSelect.h"
#include "GLFoundation.h"
#include <math.h>
#include "MapSegments.h"
#include <stdio.h>
#include "TextWriter.h"
#include "SDL.h"
#include <string.h>
#include "TrackEdit.h"
#include "Stack.h"

#define SEGHWIDTH	0.03f

int MouseOver = -1;
void DrawSegment(float x1, float y1, float ydir, int id)
{
	float sidevec[2], mul;

	sidevec[0] = -ydir;
	sidevec[1] = 0.2;
	mul = SEGHWIDTH / sqrt(sidevec[0]*sidevec[0] + sidevec[1]*sidevec[1]);
	sidevec[0] *= mul;
	sidevec[1] *= mul;

	if(Paths[id].Selected)
	{
		if(MouseOver == id)
			glColor3f(1, 1, 1);
		else
			glColor3f(1, 1, 0);
		glBegin(GL_QUADS);
	}
	else
	{
		if(MouseOver == id)
		{
			glColor3f(0, 1, 1);
			glBegin(GL_QUADS);
		}
		else
		{
			glColor3f(1, 1, 1);
			glBegin(GL_LINE_LOOP);
		}
	}

		glVertex2f(x1-sidevec[0], y1-sidevec[1]);
		glVertex2f(x1+0.2-sidevec[0], y1+ydir-sidevec[1]);
		glVertex2f(x1+0.2+sidevec[0], y1+ydir+sidevec[1]);
		glVertex2f(x1+sidevec[0], y1+sidevec[1]);
	glEnd();
}

bool MouseIn(float x1, float y1, float ydir, float mx, float my)
{
	float sidevec[2], mul;

	sidevec[0] = -ydir;
	sidevec[1] = 0.2;
	mul = 1.0f / sqrt(sidevec[0]*sidevec[0] + sidevec[1]*sidevec[1]);
	sidevec[0] *= mul;
	sidevec[1] *= mul;

	float dist = (mx - x1)*sidevec[0] + (my - y1)*sidevec[1];
	if(fabs(dist) > SEGHWIDTH) return false;

	dist = (mx - x1)*0.2f + (my-y1)*ydir;
	if(dist > (0.2f*0.2f + ydir*ydir)) return false;
	if(dist < 0) return false;

	return true;
}

void DrawPair(float x, float y, int idbase, int dir)
{
	DrawSegment(x, y, -0.15, idbase);
	DrawSegment(x, y, 0.15, idbase+dir);
}

int MouseInPair(float x, float y, int idbase, int dir, float mx, float my)
{
	if(MouseIn(x, y, -0.15, mx, my)) return idbase;
	if(MouseIn(x, y, 0.15, mx, my)) return idbase+dir;
	return -1;
}

void DrawHalfGrid(int id, int dir)
{
	DrawSegment(-0.8, 0, 0, id);

	DrawPair(-0.6, 0, id+dir, dir);

	DrawPair(-0.4, 0.15, id+dir*5, dir);
	DrawPair(-0.4, -0.15, id+dir*3, dir);

	DrawPair(-0.2, 0.3, id+dir*11, dir);
	DrawPair(-0.2, 0, id+dir*9, dir);
	DrawPair(-0.2, -0.3, id+dir*7, dir);
}

int MouseInHalfGrid(int id, int dir, float mx, float my)
{
	if(MouseIn(-0.8, 0, 0, mx, my)) return id;

	int res;
	if((res = MouseInPair(-0.6, 0, id+dir, dir, mx, my)) != -1) return res;

	if((res = MouseInPair(-0.4, 0.15, id+dir*5, dir, mx, my)) != -1) return res;
	if((res = MouseInPair(-0.4, -0.15, id+dir*3, dir, mx, my)) != -1) return res;

	if((res = MouseInPair(-0.2, 0.3, id+dir*11, dir, mx, my)) != -1) return res;
	if((res = MouseInPair(-0.2, 0, id+dir*9, dir, mx, my)) != -1) return res;
	if((res = MouseInPair(-0.2, -0.3, id+dir*7, dir, mx, my)) != -1) return res;

	return -1;
}

bool ValidRoute()
{
	if(!Paths[0].Selected) return false;
	int Total = 0;
	int c = 26;
	while(c--)
		if(Paths[c].Selected) Total++;
	if(Total != 8) return false;

	int Pos = 0;
	while(Pos != 25)
	{
		if(Paths[Pos].Next[1] >= 0 && Paths[Pos].Next[0] >= 0)
		{
			if(!Paths[Paths[Pos].Next[0]].Selected && !Paths[Paths[Pos].Next[1]].Selected) return false;
			if(Paths[Paths[Pos].Next[0]].Selected && Paths[Paths[Pos].Next[1]].Selected) return false;
			Pos = Paths[Paths[Pos].Next[0]].Selected ? Paths[Pos].Next[0] : Paths[Pos].Next[1];
		}
		else
		{
			int Real = (Paths[Pos].Next[0] != -1) ? Paths[Pos].Next[0] : Paths[Pos].Next[1];
			if(!Paths[Real].Selected) return false;
			Pos = Real;
		}
	}
	return true;
}

bool CSegmentSelect::Message(DialogueMessage *Msg)
{
	switch(Msg->Type)
	{
		default: break;
		case DialogueMessage::ENTER:
			EditButton.SetPos(1, ScrH);
			EditButton.SetText("Edit Route...");
			EditButton.SizeToFit(OT_BOTTOMRIGHT);
			EditButton.SetActive(ValidRoute());
		return true;
		case DialogueMessage::CLICK:
			if((Msg->Buttons&1) && (Msg->Data.ClickChange&1) && (MouseOver != -1))
			{
				Paths[MouseOver].Selected ^= true;
				EditButton.SetActive(ValidRoute());\
				return true;
			}
		break;
		case DialogueMessage::KEYPRESS:
			if(MouseOver != -1)
			{
				int Strlen = strlen(Paths[MouseOver].Name);
				switch(Msg->Data.Key >> 16)
				{
					case SDLK_BACKSPACE:
						if(Strlen)
							Paths[MouseOver].Name[Strlen-1] = '\0';
					break;

					default:
						if(Strlen < 32)
						{
							Paths[MouseOver].Name[Strlen] = Msg->Data.Key&255;
							Strlen++;
						}
					break;
				}
			}
		return true;
		case DialogueMessage::MOUSEMOVE:
			MouseOver = MouseInHalfGrid(0, 1, Msg->x, Msg->y);
			if(MouseOver == -1)
				MouseOver = MouseInHalfGrid(25, -1, -Msg->x, Msg->y);
		break;
		case DialogueMessage::DOTASK:
			CTrackEdit *NE = new CTrackEdit;
			PushDialogue(NE);
			DoStackOp(SI_DESCEND);
		return true;
		case DialogueMessage::DRAW:
			DisableTextures();
			DrawHalfGrid(0, 1);

			glPushMatrix();

				glScalef(-1, 1, 1);
				DrawHalfGrid(25, -1);

			glPopMatrix();

			glColor3f(1, 1, 1);

			if(MouseOver != -1)
			{
//				glEnable(GL_TEXTURE_2D);
				glColor3f(1, 1, 1);
				TextWriter->Write(-1, ScrH - (1.0f / 40.0f), Paths[MouseOver].Name);
			}
		break;
	}

	return CDialogueBase::Message(Msg);
}

CSegmentSelect::CSegmentSelect()
{
	AddWidget(&EditButton);
	EditButton.SetAction(this, 0);
}
