#include "TrackEdit.h"
#include "MapSegments.h"
#include <stdio.h>
#include <string.h>
#include "TextWriter.h"
#include "GLFoundation.h"
#include "3dview.h"

int ScrollBase;

CTrackDraw::CTrackDraw()
{
	AddWidget(&TrackScroll);
	BaseRow = 0;
	Dragging = false;
	TrackScroll.SetValue(1);
}

void CTrackDraw::SetPos(float nx1, float ny1, float nx2, float ny2)
{
	CWidgetBase::SetPos(nx1, ny1, nx2, ny2);

	/* fix scroll bar */
	TrackScroll.SetPos(x1, y1, x2, y2);
	TrackScroll.SizeToFit(OT_RIGHT);

	/* determine own stuff */
	CentreX = (x1+x2-TrackScroll.Width())*0.5f;
	GridHeight = (int)((y2 - y1) / TRACK_SCALE);
	ScrollArea = SEGMENT_LENGTH*8 - GridHeight;	/* *8 as all routes contain 8 segments */
}

void CTrackDraw::GetClickPosition(float xin, float yin, float &xout, float &yout)
{
	yout = ((y2 - yin) / TRACK_SCALE) + ScrollBase;
	xout = (xin - CentreX) / TRACK_SCALE;
}

float CTrackDraw::GetClickRow(float yin)
{
	return ((y2 - yin) / TRACK_SCALE) + ScrollBase;
}

bool CTrackDraw::Message(DialogueMessage *Msg)
{
	CDialogueBase::Message(Msg);

	switch(Msg->Type)
	{
		default: break;
		case DialogueMessage::SCROLL:
			float Add = (float)Msg->Data.Scroll / ScrollArea;
			TrackScroll.SetValue(TrackScroll.GetValue() + Add);
		return false;
		case DialogueMessage::CLICK:
			if(!Dragging)
			{
				if(Msg->x > x2 - TrackScroll.Width()) return false;
				if(Msg->x < x1) return false;
				if(Msg->y < y1) return false;
				if(Msg->y > y2) return false;
			}

			if(Msg->Buttons&2)
			{
				BaseRow = (int)GetClickRow(Msg->y);
				return true;
			}
			if(Msg->Data.ClickChange&4)
			{
				if(Msg->Buttons&4)
				{
					StartRow = EndRow = (int)GetClickRow(Msg->y);
					Dragging = true;
				}
				else
				{
					if(EndRow < StartRow)
					{
						int t = EndRow; EndRow = StartRow; StartRow = t;
					}
					int CopyRow = BaseRow;
					for(int y = StartRow; y <= EndRow; y++)
					{
						memcpy(Paths[SectionIndices[y/SEGMENT_LENGTH]].MaterialMap[y%SEGMENT_LENGTH], Paths[SectionIndices[CopyRow/SEGMENT_LENGTH]].MaterialMap[CopyRow%SEGMENT_LENGTH], 8*sizeof(Paths[0].MaterialMap[0][0]));
						CopyRow++;
					}
					Dragging = false;
				}
				return true;
			}
		break;
		case DialogueMessage::MOUSEMOVE:
			if(
				Msg->x <= x2 - TrackScroll.Width() &&
				Msg->x >= x1 &&
				Msg->y >= y1 &&
				Msg->y <= y2)
			{
				float t;
				GetClickPosition(Msg->x, Msg->y, t, MouseYPos);
			}
		
			if(Dragging)
				EndRow = (int)GetClickRow(Msg->y);
		break;
		case DialogueMessage::DOTASK:
		{
			if(Msg->x > x2 - TrackScroll.Width()) return false;
			if(Msg->x < x1) return false;
			if(Msg->y < y1) return false;
			if(Msg->y > y2) return false;

			float PosX, PosY;
			GetClickPosition(Msg->x, Msg->y, PosX, PosY);
			switch(Msg->Data.Task.Num)
			{
				case CTD_PUTTILE:
					if(PosX > -4 && PosX < 4)	/* x = 0 is the middle of the road */
					{
						Paths[SectionIndices[((int)PosY)/SEGMENT_LENGTH]].MaterialMap[((int)PosY)%SEGMENT_LENGTH][(int)(PosX+4)] = Msg->Data.Task.Data | FlipFlags;
						return true;
					}
				break;
				case CTD_PUTOBJECT:
				{
					Object *N = new Object;

					N->ImageId = Msg->Data.Task.Data | FlipFlags;
					N->x = PosX;
					N->y = PosY;

					int YOffs = ((int)N->y) / SEGMENT_LENGTH;
					Paths[SectionIndices[YOffs]].AddObject(N);
					SceneViewer->FixObjects(SectionIndices[YOffs]);
					return true;
				}
				break;
				case CTD_DRAWOBJECT:
				{
//					glEnable(GL_TEXTURE_2D);
					glColor4f(1, 1, 1, 0.5f);
					CBitmap *Img = ObjectArray->GetImage(Msg->Data.Task.Data&TILE_MASK);
					DrawObject(Img, Msg->x, Msg->y, Msg->Data.Task.Data | FlipFlags, TRACK_SCALE);
					return true;
				}
				break;
				case CTD_TESTOBJECT:
				{
					Object *R = NULL;

					int TopSeg = (ScrollBase+GridHeight)/SEGMENT_LENGTH;
					if(TopSeg > 7) TopSeg = 7;

					Object *FirstObject = Paths[SectionIndices[TopSeg]].ObjList;
					while(FirstObject && FirstObject->y > (ScrollBase+GridHeight))
						FirstObject = FirstObject->Next;
					while(FirstObject && FirstObject->y > (ScrollBase - 8))	/* to allow objects up to 512 pixels high to appear */
					{
						CBitmap *Img = ObjectArray->GetImage(FirstObject->ImageId&TILE_MASK);

						if(PointOverObject(Img, CentreX + FirstObject->x*TRACK_SCALE, y2 - (FirstObject->y - ScrollBase)*TRACK_SCALE, Msg->x, Msg->y, FirstObject->ImageId, TRACK_SCALE))
							R = FirstObject;

						FirstObject = FirstObject->Next;
					}

					if((ScrollBase%SEGMENT_LENGTH) >= (SEGMENT_LENGTH-GridHeight) || ((ScrollBase%SEGMENT_LENGTH) <= 8 && ScrollBase > 8))
					{
						FirstObject = Paths[SectionIndices[TopSeg-1]].ObjList;

						while(FirstObject && FirstObject->y > (ScrollBase - 8))	/* to allow objects up to 512 pixels high to appear */
						{
							CBitmap *Img = ObjectArray->GetImage(FirstObject->ImageId&TILE_MASK);
							if(PointOverObject(Img, CentreX + FirstObject->x*TRACK_SCALE, y2 - (FirstObject->y - ScrollBase)*TRACK_SCALE, Msg->x, Msg->y, FirstObject->ImageId, TRACK_SCALE))
								R = FirstObject;

							FirstObject = FirstObject->Next;
						}
					}

					if(R)
					{
						Msg->Data.Task.Ptr = (void *)R;
						return true;
					}
					else
						return false;
				}
				break;

				default: return false;
			}
		}
		break;
		case DialogueMessage::DRAW:
			GLFoundation.SetScissor(x1, y1, x2 - TrackScroll.Width(), y2);
			int y = (int)((1.0f - TrackScroll.GetValue())*ScrollArea);
			ScrollBase = y;

			/* draw marks for any visible corners */
			DisableTextures();
			glColor4f(1, 1, 0, 0.25f);

			int SY = ScrollBase/SEGMENT_LENGTH;
			CornerNode *C = Paths[SectionIndices[SY]].CornerList;
			glBegin(GL_QUADS);
			while(C && C->y < ScrollBase)
				C = C->Next;
			while(C && C->y < ScrollBase+GridHeight)
			{
				float transy = y2 - (C->y - ScrollBase)*TRACK_SCALE;
				glVertex2f(x1, transy-0.01f); glVertex2f(x2, transy-0.01f);
				glVertex2f(x2, transy+0.01f); glVertex2f(x1, transy+0.01f);
				
				C = C->Next;
			}
			if((ScrollBase%SEGMENT_LENGTH) >= (SEGMENT_LENGTH-GridHeight))
			{
				SY++;
				if(SY < 8)
				{
					C = Paths[SectionIndices[SY]].CornerList;
					while(C && C->y < ScrollBase+GridHeight)
					{
						float transy = y2 - (C->y - ScrollBase)*TRACK_SCALE;
						glVertex2f(x1, transy-0.01f); glVertex2f(x2, transy-0.01f);
						glVertex2f(x2, transy+0.01f); glVertex2f(x1, transy+0.01f);

						C = C->Next;
					}
				}
			}
			glEnd();

			/* proceed to tile map */
			glColor3f(1, 1, 1);

			float CurY = y2;
			while(CurY > y1 && y<SEGMENT_LENGTH*8)
			{
				/* guide line */
				if(!(y&15))
				{
					DisableTextures();
					glColor4f(0, 0, 1, 0.5);
					glBegin(GL_LINES);
						glVertex2f(x1, CurY);
						glVertex2f(x2-TrackScroll.Width(), CurY);
					glEnd();
					glColor3f(1, 1, 1);
				}

				/* actual tiles */
				for(int x = 0; x < 8; x++)
				{
					int TileNum = Paths[SectionIndices[y/SEGMENT_LENGTH]].MaterialMap[y%SEGMENT_LENGTH][x];

					if(TileNum != 0xffff)
					{
						MaterialArray->GetImage(TileNum&TILE_MASK)->Activate();
						glBegin(GL_QUADS);
							glTexCoord2f((TileNum&TILE_HFLIP) ? 1 : 0, (TileNum&TILE_VFLIP) ? 0 : 1); glVertex2f(CentreX + (x-4)*TRACK_SCALE, CurY);
							glTexCoord2f((TileNum&TILE_HFLIP) ? 0 : 1, (TileNum&TILE_VFLIP) ? 0 : 1); glVertex2f(CentreX + (x-3)*TRACK_SCALE, CurY);
							glTexCoord2f((TileNum&TILE_HFLIP) ? 0 : 1, (TileNum&TILE_VFLIP) ? 1 : 0); glVertex2f(CentreX + (x-3)*TRACK_SCALE, CurY-TRACK_SCALE);
							glTexCoord2f((TileNum&TILE_HFLIP) ? 1 : 0, (TileNum&TILE_VFLIP) ? 1 : 0); glVertex2f(CentreX + (x-4)*TRACK_SCALE, CurY-TRACK_SCALE);
						glEnd();
					}
				}

				/* consider drawing section border */
				if(y && !(y%SEGMENT_LENGTH))
				{
					int Dir = 0;
					if(Paths[SectionIndices[(y-1)/SEGMENT_LENGTH]].Next[0] == -1) Dir |= 1;
					if(Paths[SectionIndices[(y-1)/SEGMENT_LENGTH]].Next[1] == -1) Dir |= 2;
					if(Paths[SectionIndices[(y-1)/SEGMENT_LENGTH]].Next[0] == SectionIndices[y/SEGMENT_LENGTH]) Dir |= 1;
					if(Paths[SectionIndices[(y-1)/SEGMENT_LENGTH]].Next[1] == SectionIndices[y/SEGMENT_LENGTH]) Dir |= 2;
		
					DisableTextures();
					glColor4f(1, 0, 1, 0.5f);
					glBegin(GL_QUADS);
						glVertex2f((Dir&1) ? x1 : CentreX, CurY-0.01f);
						glVertex2f((Dir&2) ? x2-TrackScroll.Width() : CentreX, CurY-0.01f);
						glVertex2f((Dir&2) ? x2-TrackScroll.Width() : CentreX, CurY+0.01f);
						glVertex2f((Dir&1) ? x1 : CentreX, CurY+0.01f);
					glEnd();
					glColor3f(1, 1, 1);
				}

				/* base row */
				if(y == BaseRow)
				{
					DisableTextures();
					glColor4f(1, 1, 1, 0.5f);
					glBegin(GL_QUADS);
						glVertex2f(x1, CurY);
						glVertex2f(x2-TrackScroll.Width(), CurY);
						glVertex2f(x2-TrackScroll.Width(), CurY-TRACK_SCALE);
						glVertex2f(x1, CurY-TRACK_SCALE);
					glEnd();
					glColor3f(1, 1, 1);
				}

				/* drag area */
				if(Dragging && 
					(
						(y >= StartRow && y <= EndRow) ||
						(y <= StartRow && y >= EndRow)
					)
				)
				{
					DisableTextures();
					glColor4f(1, 0, 1, 0.5f);
					glBegin(GL_QUADS);
						glVertex2f(x1, CurY);
						glVertex2f(x2-TrackScroll.Width(), CurY);
						glVertex2f(x2-TrackScroll.Width(), CurY-TRACK_SCALE);
						glVertex2f(x1, CurY-TRACK_SCALE);
					glEnd();
					glColor3f(1, 1, 1);
				}

				CurY -= TRACK_SCALE;
				y++;
			}

			/* actual grid lines */
			DisableTextures();
			glColor4f(1, 1, 1, 0.1f);
			glBegin(GL_LINES);
			for(int x = 0; x < 9; x++)
			{
				float fx = CentreX + (x-4)*TRACK_SCALE;
				glVertex2f(fx, y1);
				glVertex2f(fx, y2);
			}
			CurY = y2;
			while(CurY > y1)
			{
				glVertex2f(CentreX - 4*TRACK_SCALE, CurY);
				glVertex2f(CentreX + 4*TRACK_SCALE, CurY);
				CurY -= TRACK_SCALE;
			}
			
				/* ypos line */
				glColor4f(1, 0, 1, 0.2f);
				float MY = y2 - (MouseYPos - ScrollBase)*TRACK_SCALE;
				glVertex2f(x1, MY);
				glVertex2f(x2, MY);
			
			glEnd();

			/* objects */
			int TopSeg = (ScrollBase+GridHeight)/SEGMENT_LENGTH;
			if(TopSeg > 7) TopSeg = 7;

			Object *FirstObject = Paths[SectionIndices[TopSeg]].ObjList;
			while(FirstObject && FirstObject->y > (ScrollBase+GridHeight))
				FirstObject = FirstObject->Next;
			while(FirstObject && FirstObject->y > (ScrollBase - 8))	/* to allow objects up to 512 pixels high to appear */
			{
				CBitmap *Img = ObjectArray->GetImage(FirstObject->ImageId&TILE_MASK);

				if(MouseOverObject == FirstObject) glColor4f(0, 1, 1, 0.5); else glColor3f(1, 1, 1);
				DrawObject(Img, CentreX + FirstObject->x*TRACK_SCALE, y2 - (FirstObject->y - ScrollBase)*TRACK_SCALE, FirstObject->ImageId, TRACK_SCALE);

				FirstObject = FirstObject->Next;
			}

			if((ScrollBase%SEGMENT_LENGTH) >= (SEGMENT_LENGTH-GridHeight) || ((ScrollBase%SEGMENT_LENGTH) <= 8 && ScrollBase > 8))
			{
				FirstObject = Paths[SectionIndices[TopSeg-1]].ObjList;

				while(FirstObject && FirstObject->y > (ScrollBase - 8))	/* to allow objects up to 512 pixels high to appear */
				{
					CBitmap *Img = ObjectArray->GetImage(FirstObject->ImageId&TILE_MASK);

					if(MouseOverObject == FirstObject) glColor4f(0, 1, 1, 0.5); else glColor3f(1, 1, 1);
					DrawObject(Img, CentreX + FirstObject->x*TRACK_SCALE, y2 - (FirstObject->y - ScrollBase)*TRACK_SCALE, FirstObject->ImageId, TRACK_SCALE);

					FirstObject = FirstObject->Next;
				}
			}

			glColor3f(1, 1, 1);
			TextWriter->Write(x1, y1, Paths[SectionIndices[TopSeg]].Name);
			GLFoundation.DisableScissor();
		return false;
	}

	return false;
}
