#include "TrackEdit.h"
#include "GLFoundation.h"
#include "TextWriter.h"
#include "Stack.h"
#include <stdio.h>
#include "MapSegments.h"
#include "ImageArray.h"
#include <string.h>
#include "3dView.h"

int SectionIndices[8];
Object *MouseOverObject;
int FlipFlags = 0;

/*

	TRACKEDIT

*/

#define	BACK_BUTTON		0
#define	MATERIAL_SELECT	1
#define OBJECT_SELECT	2

float MouseYPos;

CTrackEdit::CTrackEdit()
{
	AddWidget(&BackButton);
	BackButton.SetAction(this, BACK_BUTTON);

	AddWidget(&MaterialBrowser);
	AddWidget(&ObjectBrowser);

	MaterialBrowser.SetImageSet(MaterialArray);
	ObjectBrowser.SetImageSet(ObjectArray);

	MaterialBrowser.SetAction(this, MATERIAL_SELECT);
	ObjectBrowser.SetAction(this, OBJECT_SELECT);
	ThingId = -1;

	AddWidget(&TrackView);
	AddWidget(&ElevationEditor);
	AddWidget(&PreviewArea);
}

bool CTrackEdit::Message(DialogueMessage *Msg)
{
	if(Msg->Type != DialogueMessage::ENTER) CDialogueBase::Message(Msg);
	switch(Msg->Type)
	{
		default: break;
		case DialogueMessage::ENTER:
		{
			MouseOverObject = NULL;

			BackButton.SetPos(-ScrW, ScrH);
			BackButton.SetText("Back...");
			BackButton.SizeToFit(OT_BOTTOMLEFT);

			TrackView.SetPos(-ScrW, -ScrH/2, 0, ScrH);
			ElevationEditor.SetPos(-ScrW, -ScrH, ScrW, -ScrH/2);
			PreviewArea.SetPos(0, ScrH - ((ScrW*3)/4), ScrW, ScrH);

			MaterialBrowser.SetPos(0, -ScrH/2, ScrW*0.5f, PreviewArea.Top());
			ObjectBrowser.SetPos(ScrW*0.5f, -ScrH/2, ScrW, PreviewArea.Top());

			SectionIndices[0] = 0;
			int Pos = 0;
			while(Pos < 7)
			{
				if(Paths[SectionIndices[Pos]].Next[0] != -1 && Paths[Paths[SectionIndices[Pos]].Next[0]].Selected)
				{
					SectionIndices[Pos+1] = Paths[SectionIndices[Pos]].Next[0];
					Pos++;
				}
				else
				{
					SectionIndices[Pos+1] = Paths[SectionIndices[Pos]].Next[1];
					Pos++;
				}
			}
			CDialogueBase::Message(Msg);
		}
		return true;
		case DialogueMessage::DOTASK:
			switch(Msg->Data.Task.Num)
			{
				case BACK_BUTTON:	DoStackOp(SI_ASCEND);	return true;
				case MATERIAL_SELECT:
					ObjectBrowser.SetSelected(-1);
					ThingId = Msg->Data.Task.Data;
					MaterialMode = true;
				return true;
				case OBJECT_SELECT:
					MaterialBrowser.SetSelected(-1);
					ThingId = Msg->Data.Task.Data;
					MaterialMode = false;
				return true;
			}
		break;
		case DialogueMessage::CLICK:
		case DialogueMessage::MOUSEMOVE:
			if(ThingId == -1)
			{
				/* see if we're over an object */
				DialogueMessage NewMsg;
				NewMsg.Type = DialogueMessage::DOTASK;
				NewMsg.Data.Task.Num = CTD_TESTOBJECT;
				NewMsg.x = Msg->x;
				NewMsg.y = Msg->y;
				if(TrackView.Message(&NewMsg))
					MouseOverObject = (Object *)NewMsg.Data.Task.Ptr;
				else
					MouseOverObject = NULL;
			}
			if(Msg->Buttons&1)
			{
				if(!MouseOverObject)
				{
					DialogueMessage NewMsg;
					NewMsg.x = Msg->x;
					NewMsg.y = Msg->y;
					NewMsg.Type = DialogueMessage::DOTASK;
					NewMsg.Data.Task.Num = (MaterialMode || ThingId == -1) ? CTD_PUTTILE : CTD_PUTOBJECT;
					NewMsg.Data.Task.Data = ThingId;

					if(Msg->Type == DialogueMessage::CLICK || NewMsg.Data.Task.Num == CTD_PUTTILE)
						TrackView.Message(&NewMsg);
				}
				else
					if(Msg->Type == DialogueMessage::CLICK)
					{
						ThingId = MouseOverObject->ImageId;
						MaterialMode = false;

						int idx = ScrollBase/SEGMENT_LENGTH;
						if(!Paths[SectionIndices[idx]].RemoveObject(MouseOverObject))
							Paths[SectionIndices[idx+1]].RemoveObject(MouseOverObject);
						MouseOverObject = NULL;
						SceneViewer->FixObjects(SectionIndices[idx]);
						SceneViewer->FixObjects(SectionIndices[idx+1]);
					}
			}
		break;
		case DialogueMessage::KEYPRESS:
			switch((Msg->Data.Key >> 16))
			{
				case SDLK_SPACE:
					ObjectBrowser.SetSelected(-1);
					MaterialBrowser.SetSelected(-1);
					ThingId = -1;
				break;
				
				default: break;
				
				case SDLK_LEFT:
				case SDLK_RIGHT:
					FlipFlags ^= TILE_HFLIP;
				break;
				case SDLK_UP:
				case SDLK_DOWN:
					FlipFlags ^= TILE_VFLIP;
				break;
			}
		break;
		case DialogueMessage::DRAW:
			if(!MaterialMode && ThingId != -1 &&
				Msg->x >= TrackView.Left() &&
				Msg->x <= TrackView.Right() &&
				Msg->y >= TrackView.Top() &&
				Msg->y <= TrackView.Bottom()
			)
			{
				DialogueMessage NewMsg;
				NewMsg.Type = DialogueMessage::DOTASK;
				NewMsg.x = Msg->x;
				NewMsg.y = Msg->y;
				NewMsg.Data.Task.Num = CTD_DRAWOBJECT;
				NewMsg.Data.Task.Data = ThingId;
				TrackView.Message(&NewMsg);
			}

			/* divider lines */
			DisableTextures();
			glColor3f(1, 1, 1);
			glBegin(GL_LINES);
				glVertex2f(-ScrW, TrackView.Top()); glVertex2f(ScrW, TrackView.Top());
				glVertex2f(TrackView.Right(), TrackView.Top()); glVertex2f(TrackView.Right(), TrackView.Bottom());
				glVertex2f(ObjectBrowser.Left(), ObjectBrowser.Top()); glVertex2f(ObjectBrowser.Left(), ObjectBrowser.Bottom());
				glVertex2f(PreviewArea.Left(), PreviewArea.Top()); glVertex2f(PreviewArea.Right(), PreviewArea.Top());
			glEnd();

			/* text labels */
//			glEnable(GL_TEXTURE_2D);
			TextWriter->Write(MaterialBrowser.Left(), MaterialBrowser.Top(), "Floor Tiles");
			TextWriter->Write(ObjectBrowser.Left(), ObjectBrowser.Top(), "Objects");
			TextWriter->Write(ElevationEditor.Left(), ElevationEditor.Top(), "Elevation");
			TextWriter->Write(PreviewArea.Left(), PreviewArea.Top(), "Preview");
		return false;
	}

	return false;
}

void DrawObject(CBitmap *Img, float x, float y, int f, float scale)
{
	float fw, fh;

	fw = (Img->Width() / 128.0f) * scale;
	fh = (Img->Height() / 128.0f) * scale;

	Img->Draw(x - fw*0.5f, y-fh, x + fw*0.5f, y, f);
}

bool PointOverObject(CBitmap *Img, float objx, float objy, float mx, float my, int f, float scale)
{
	float fw, fh, flx, fly;

	fw = (Img->Width() / 128.0f) * scale;
	fh = (Img->Height() / 128.0f) * scale;

	flx = (mx - (objx - fw*0.5f)) / fw;
	fly = (my - (objy - fh)) / fh;

	if(flx < 0) return false;
	if(flx > 1) return false;
	if(fly < 0) return false;
	if(fly > 1) return false;

	if(f&BMP_HFLIP) flx = 1.0f - flx;
	if(f&BMP_VFLIP) fly = 1.0f - fly;

	return Img->PointOver(flx, fly);
}
