#ifndef __TRACKEDIT_H
#define __TRACKEDIT_H

#define	TRACK_SCALE		0.035f

#include "DialogueBase.h"
#include "Widgets.h"
#include "ImageArray.h"
#include "MapSegments.h"

extern int SectionIndices[8];
extern void DrawObject(CBitmap *Img, float x, float y, int f, float scale = TRACK_SCALE);
extern bool PointOverObject(CBitmap *Img, float objx, float objy, float mx, float my, int f, float scale = TRACK_SCALE);
extern int ScrollBase;
extern float MouseYPos;
extern Object *MouseOverObject;
extern int FlipFlags;

class CImageBrowser: public CWidgetBase
{
	public:
		CImageBrowser();

		bool Message(DialogueMessage *Msg);
		void SetPos(float x1, float y1, float x2, float y2);
		void SetImageSet(CImageArray *imga);
		
		void SetSelected(int d) {Selected = d;}

	private:
		CSlider PositionSlider;
		CImageArray *IArray;
		int base, Selected;
};

class CElevationEditor: public CWidgetBase
{
	public:
		CElevationEditor();
		bool Message(DialogueMessage *Msg);
		void SetPos(float x1, float y1, float x2, float y2);

	private:
		void DoCurve(ElevationNode *a, ElevationNode *b);
		void DrawCornerMark(float x, CornerNode *Src);

		void StartCurve(ElevationNode *a, ElevationNode *b);
		void AdvanceCurve(ElevationNode *a, ElevationNode *b);

		ElevationNode *GetElevationNode(float x, float y, int &dir);
		CornerNode *GetCornerNode(float x, float y, int &dir);

		float GetArcPos(float y);

		float x[2], y[2];
		float CurLength, SectionLength;
		int CurvePos;

		float StartY;

		float CentreY;
		ElevationNode *DragElevation;
		CornerNode *DragCorner;
		MapSegment *Segment;
		int DragTarget;
};

class C3dPreview: public CWidgetBase
{
	public:
		bool Message(DialogueMessage *Msg);
		void SetPos(float x1, float y1, float x2, float y2);
};

#define	CTD_PUTTILE		0
#define CTD_PUTOBJECT	1
#define CTD_DRAWOBJECT	2
#define CTD_TESTOBJECT	3

class CTrackDraw: public CWidgetBase
{
	public:
		CTrackDraw();

		bool Message(DialogueMessage *Msg);
		void SetPos(float x1, float y1, float x2, float y2);

		void GetClickPosition(float xin, float yin, float &xout, float &yout);
		float GetClickRow(float yin);
	private:
		CSlider TrackScroll;
		float CentreX;

		int GridHeight, ScrollArea;

		int BaseRow, StartRow, EndRow;
		bool Dragging;
};

class CTrackEdit: public CDialogueBase
{
	public:
		CTrackEdit();

		bool Message(DialogueMessage *Msg);

	private:
		CButton BackButton;
		CImageBrowser MaterialBrowser, ObjectBrowser;
		CTrackDraw TrackView;
		CElevationEditor ElevationEditor;
		C3dPreview PreviewArea;
		
		int ThingId;
		bool MaterialMode;
};

#endif
