#include "Widgets.h"
#include "TextWriter.h"
#include "GLFoundation.h"
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>

/*

	CBUTTON

*/

#define BUTTON_BORDER	0.03f

CButton::CButton()
{
	Label = NULL;
	Parent = NULL;
	MouseOver = false;
	Active = true;
}

CButton::~CButton()
{
	if(Label) free(Label);
}

void CButton::SetText(char *t)
{
	if(Label) free(Label);
	Label = strdup(t);
}

void CButton::SetPos(float nx1, float ny1)
{
	x1 = nx1; y1 = ny1;
}

void CButton::SizeToFit(OrientationFour o)
{
	switch(o)
	{
		case OT_TOPLEFT:
			x2 = BUTTON_BORDER + x1 + TextWriter->GetLength(Label);
			y2 = BUTTON_BORDER + y1 + 1.0f / 40.0f;
		break;
		
		case OT_BOTTOMLEFT:
			x2 = BUTTON_BORDER + x1 + TextWriter->GetLength(Label);
			y2 = y1; y1 = y2 - BUTTON_BORDER - 1.0f / 40.0f;
		break;

		case OT_BOTTOMRIGHT:
			x2 = x1; x1 = x2 - BUTTON_BORDER - TextWriter->GetLength(Label);
			y2 = y1; y1 = y2 - BUTTON_BORDER - 1.0f / 40.0f;
		break;

		case OT_TOPRIGHT:
			x2 = x1; x1 = x2 - BUTTON_BORDER - TextWriter->GetLength(Label);
			y2 = BUTTON_BORDER + y1 + 1.0f / 40.0f;
		break;
	}
}

void CButton::SetActive(bool b)
{
	Active = b;
}

bool CButton::Message(DialogueMessage *Msg)
{
	switch(Msg->Type)
	{
		default: return false;
	
		case DialogueMessage::MOUSEMOVE:
			MouseOver =
					Msg->x >= x1 && Msg->x <= x2  &&
					Msg->y >= y1 && Msg->y <= y2;
		return false;

		case DialogueMessage::CLICK:
			if(Active && MouseOver && Parent && Msg->Buttons)
			{
				DialogueMessage NewMsg;
				NewMsg.Type = DialogueMessage::DOTASK;
				NewMsg.Data.Task.Num = id;
				Parent->Message(&NewMsg);
			}
		return true;

		case DialogueMessage::DRAW:
			DisableTextures();
	
			if(MouseOver && Active)
			{
				float add = sin(3.141592654f*2.0f*(float)(SDL_GetTicks()&4095) / 4095.0f);
				glColor3f(0, 0, 0.4f + add*0.4f);
			}
			else
				glColor3f(0.2f, 0.2, 0.2f);

			glBegin(GL_QUADS);
				glVertex2f(x1, y1);
				glVertex2f(x2, y1);
				glVertex2f(x2, y2);
				glVertex2f(x1, y2);
			glEnd();
	
			if(Active) glColor3f(1, 1, 1); else glColor3f(0.4f, 0.4f, 0.4f);
			TextWriter->Write(x1 + BUTTON_BORDER*0.5f, y1 + BUTTON_BORDER*0.5f, Label);
		return false;
	}
}

/*

	CSLIDER

*/

CSlider::CSlider()
{
/*	AddWidget(&LeftButton);
	AddWidget(&RightButton);

	LeftButton.SetText("<");
	RightButton.SetText(">");
	
	LeftButton.SetAction(this, -1);
	RightButton.SetAction(this, 1);*/
	
	Value = 0.5f;
	GrabMouse = false;
}

void CSlider::SetPos(float nx1, float ny1, float nx2, float ny2)
{
	x1 = nx1; x2 = nx2; y1 = ny1; y2 = ny2;

//	LeftButton.SetPos(x1, y1); LeftButton.SizeToFit(OT_TOPLEFT);
//	RightButton.SetPos(x2, y2); RightButton.SizeToFit(OT_BOTTOMRIGHT);
}

void CSlider::SizeToFit(OrientationTwo o)
{
	float Width, Height;
	
//	Width = LeftButton.Width();
//	Height = LeftButton.Height();
	Width = Height = (1.0f / 40.0f) + 0.03f;

	switch(o)
	{
		case OT_TOP:	y2 = y1 + Height; Horizontal = true;		break;
		case OT_BOTTOM:	y1 = y2 - Height; Horizontal = true;		break;
		case OT_RIGHT:	x1 = x2 - Width; Horizontal = false;		break;
		case OT_LEFT:	x2 = x1 + Width; Horizontal = false;		break;
	}
//	LeftButton.SetPos(x1, y1); LeftButton.SizeToFit(OT_TOPLEFT);
//	RightButton.SetPos(x2, y2); RightButton.SizeToFit(OT_BOTTOMRIGHT);
}

bool CSlider::Message(DialogueMessage *Msg)
{
	CDialogueBase::Message(Msg);

	switch(Msg->Type)
	{
		default:break;
		case DialogueMessage::DOTASK:
			Value += Msg->Data.Task.Num*0.01f;
			if(Value < 0) Value = 0;
			if(Value > 1) Value = 1;
		return true;
		case DialogueMessage::CLICK:
			GrabMouse = false;
			if(Msg->x > x2) return false;
			if(Msg->x < x1) return false;
			if(Msg->y > y2) return false;
			if(Msg->y < y1) return false;

			GrabMouse = Msg->Buttons ? true : false;

		case DialogueMessage::MOUSEMOVE:
			if(GrabMouse)
			{
				if(Msg->Buttons&1)
				{
					if(Horizontal)
					{
						float Width = x2 - x1 - 0.05f;
						float Start = x1 + 0.025f;
					
						Value = (Msg->x - Start) / Width;
					}
					else
					{
						float Height = y2 - y1 - 0.05f;
						float Start = y1 + 0.025f;

						Value = (Msg->y - Start) / Height;
					}
					if(Value < 0) Value = 0;
					if(Value > 1) Value = 1;
				}
			}
		break;
		case DialogueMessage::DRAW:
			DisableTextures();
			glColor3f(0.2f, 0.2f, 0.2f);
			glBegin(GL_LINE_LOOP);
				glVertex2f(x1, y1);
				glVertex2f(x2, y1);
				glVertex2f(x2, y2);
				glVertex2f(x1, y2);
			glEnd();

			if(Horizontal)
			{
				float Width = x2 - x1 - 0.05f;
				float Pos = x1 + Width*Value + 0.025f;
	
				glColor3f(1, 1, 1);
				glBegin(GL_LINES);
					glVertex2f(Pos, y1);
					glVertex2f(Pos, y2);
				glEnd();
			}
			else
			{
				float Height = y2 - y1 - 0.05f;
				float Pos = y1 + Height*Value + 0.025f;

				glColor3f(1, 1, 1);
				glBegin(GL_LINES);
					glVertex2f(x1, Pos);
					glVertex2f(x2, Pos);
				glEnd();
			}
		return false;
	}

	return false;
}
